import * as fc from 'fast-check'
import formatFileSize from './formatFileSize'

it('should exist', () => {
  expect(formatFileSize).toBeDefined()
})

it('should be a function', () => {
  expect(typeof formatFileSize).toBe('function')
})

it('should accept 2 arguments', () => {
  expect(formatFileSize).toHaveLength(2)
})

describe('on numeric arguments', () => {
  it('should format values of |x| < 1000', () => {
    fc.assert(
      fc.property(
        fc.integer().filter(v => Math.abs(v) < 1000),
        v => {
          expect(formatFileSize(v)).toMatch(/^-?\d+ B$/)
        }
      )
    )
  })

  it('should format values of |x| >= 1000', () => {
    fc.assert(
      fc.property(
        fc.integer().filter(v => Math.abs(v) >= 1000),
        v => {
          expect(formatFileSize(v)).toMatch(/^-?\d+\.\d\d .B$/)
        }
      )
    )
  })

  it('should throw an error on NaN', () => {
    expect(() => formatFileSize(NaN)).toThrow(RangeError)
  })
})

describe('on non-numeric arguments', () => {
  it('should throw an error', () => {
    fc.assert(
      fc.property(
        fc.anything().filter(v => typeof v !== 'number'),
        v => {
          expect(() => formatFileSize(v as number)).toThrow(TypeError)
        }
      )
    )
  })
})

describe('on raw', () => {
  it('should throw an error on non-numeric arguments', () => {
    fc.assert(
      fc.property(
        fc.anything().filter(v => typeof v !== 'number'),
        v => {
          expect(() => formatFileSize(v as number, true)).toThrow(TypeError)
        }
      )
    )
  })

  it('should throw an error on NaN', () => {
    expect(() => formatFileSize(NaN, true)).toThrow(RangeError)
  })

  it('should return string on numeric values', () => {
    fc.assert(
      fc.property(fc.integer(), v => {
        expect(typeof formatFileSize(v, true)).toBe('string')
      })
    )
  })

  it('should format numeric values', () => {
    fc.assert(
      fc.property(fc.integer(), v => {
        expect(formatFileSize(v, true)).toMatch(/^-?\d[,\d]* .?B$/)
      })
    )
  })
})
